/*==============================================================================
   DEMO.C :- SpellTime Demo Program
   Sub Systems, Inc.
   Software License Agreement  (1992)
   ----------------------------------
   This is a demo program.  The program shows how you can utilize SpellTime
   DLL routines to perform spell checking functions.

   Most of the programming code in this program is used to control the
   screen activities for the demo window and the user input. To avoid
   confusion, concentrate on the SpellTime function calls within the
   'CheckFile' routine.  The SpellTime functions begin with 'Spell' or
   'St' prefix.
===============================================================================*/

#include "windows.h"
#include "stdio.h"
#include "stdlib.h"
#include "fcntl.h"
#include "sys\types.h"
#include "sys\stat.h"
#include "io.h"
#include "string.h"

#if defined(__TURBOC__)
   #include "dir.h"
   #define _makepath fnmerge
   #define _splitpath fnsplit
#endif

/******************************************************************************
    WIN32 specific defines
*******************************************************************************/
#if defined (_WIN32)
   #if !defined(WIN32)
     #define WIN32
   #endif
#endif
#if defined (WIN32)
   #undef  huge
   #define huge
   #undef  FreeProcInstance
   #define FreeProcInstance(x)
   #define GET_WM_COMMAND_ID(wp,lp) LOWORD(wp)
   #define farmalloc  malloc
   #define farrealloc realloc
   #define farfree    free
#else
   #define GET_WM_COMMAND_ID(wp,lp) (wp)
#endif

/******************************************************************************
    Include SPELL.H file
*******************************************************************************/
#include "spell.h"

#include "demo.h"

/******************************************************************************
                    Windows Main Function
******************************************************************************/

int PASCAL WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    MSG msg;

    hPrevInst=hPrevInstance;

    if (lstrlen(lpCmdLine)>0) {        // check if a file specified for spell checking
      lstrcpy(InputFile,lpCmdLine);
      FileSpecified=TRUE;
    }

    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return (FALSE);

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    if (FileSpecified) PostMessage(hWnd,WM_COMMAND,IDM_FILE,0L);

    while (GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    /* update the user dictionary file */
    StResetUserDict2(SplId,NULL,NULL);
    StEndSession(SplId);

    return (int)(msg.wParam);
}

/****************************************************************************
    Process Main Window messages
****************************************************************************/
LRESULT CALLBACK _export StWndDemo(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam)
{
    FARPROC lpProc;
    int DoSpell;
    int  CommandId;

    switch (message) {
        case WM_PAINT:
             RepaintWin();                 /* repaint the window */
             break;

        case WM_COMMAND:
            // extract the command id from wParam
            CommandId=GET_WM_COMMAND_ID(wParam,lParam);
            switch (CommandId) {
               case IDM_WORD:             /* spell check a word */
                  lpProc = MakeProcInstance((FARPROC)StDlgWord,hInst);   /* call the dialog box */
                  DialogBox(hInst,"StWord",hWnd,(LPVOID)lpProc);
                  FreeProcInstance(lpProc);
                  break;

               case IDM_FILE:             /* spell check a text file */
                  if (!FileSpecified) {   // file not specified on the command line
                     lpProc = MakeProcInstance((FARPROC)StDlgFile,hInst);   /* call the dialog box */
                     DoSpell=(int)DialogBox(hInst,"StFile",hWnd,(LPVOID)lpProc);
                     FreeProcInstance(lpProc);
                  }
                  else DoSpell=TRUE;
                  if (DoSpell) CheckFile(); /* spell check the given file */

                  break;

               default:
                  return (DefWindowProc(hWnd, message, wParam, lParam));
            }
            break;

        case WM_DESTROY:
            ReleaseDC(hWnd,hDC);
            PostQuitMessage(0);
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (LRESULT)(NULL);
}

/****************************************************************************
   Initialize window data and register window class
****************************************************************************/

BOOL InitApplication(HANDLE hInstance)
{
    WNDCLASS  wc;

    wc.style = CS_DBLCLKS|CS_OWNDC|CS_HREDRAW|CS_VREDRAW;    /* allocate a DC for this class */
    wc.lpfnWndProc = (void far *)StWndDemo;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  FileSpecified?NULL:"DemoMenu";
    wc.lpszClassName = "DemoStClass";

    return (RegisterClass(&wc));
}


/****************************************************************************
    Save instance handle and create main window
****************************************************************************/

BOOL InitInstance(HANDLE hInstance,int nCmdShow)
{
    int             i;

    hInst = hInstance;

    hWnd = CreateWindow(
        "DemoStClass",
        "SpellTime Demo",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        NULL,
        NULL,
        hInstance,
        NULL
    );

    if (!hWnd)
        return (FALSE);

    if (!InitWin()) return FALSE;   /* initialize the window variables */

    if (FileSpecified) ReadOnly=FALSE;   // the file specified at the command line is always fixed for any errors
    else {
       InputFile[0]=0;    /* file to spell check */
       ReadOnly=TRUE;
    }

    strcpy(StatusLine,"");
    for (i=0;i<MAX_LINES;i++) strcpy(line[i],"");

    SetBkColor(hDC,(COLORREF)0xFF0000);
    SetTextColor(hDC,(COLORREF)0xFFFFFF);

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    hWaitCursor=LoadCursor(NULL,IDC_WAIT);    /* load hour glass cursor */

    SplId=StInitSession();                  // begin a spell checker session

    //StSetFlags2(SplId,TRUE,STFLAG_USE_APOSTROPHE);
    //StSetFlags2(SplId,TRUE,STFLAG_APOSTROPHE_JOINER);
    //StSetFlags2(SplId,TRUE,STFLAG_NO_NUM_IN_WORD);
    //StSetFlags2(SplId,TRUE,STFLAG_SPANISH_DLG);

    return (TRUE);
}

/****************************************************************************
    Process messages for "StDlgWord" dialog box.
    This dialog box spell checks individual words.
****************************************************************************/

BOOL CALLBACK _export StDlgWord(HWND hDlg,UINT message,WPARAM wParam,LPARAM lParam)
{
    char CurWord[50];
    struct StrStResult result;

    switch (message) {
        case WM_INITDIALOG:
            SetFocus(GetDlgItem(hDlg,IDC_WORD));
            return (FALSE);

        case WM_COMMAND:
            switch (wParam) {
                case IDC_WORD:
                   SetDlgItemText(hDlg,IDC_WORD_RESULT," ");
                   return TRUE;

                case IDOK:
                   GetDlgItemText(hDlg,IDC_WORD,CurWord,49);
                   //StSetFlags2(SplId,TRUE,STFLAG_FRENCH_DLG);

                   if (SpellWord2(SplId,CurWord,ST_INTERACTIVE,hDlg,(struct StrStResult far *)&result))
                        SetDlgItemText(hDlg,IDC_WORD_RESULT,"CORRECT");
                   else SetDlgItemText(hDlg,IDC_WORD_RESULT,"INCORRECT");

                   //{
                   //   char OutBuf[100],msg[200];
                   //   int  result=SpellString(CurWord,OutBuf,100,hDlg);
                   //   sprintf(msg,"%s, result: %d",OutBuf,result);
                   //   MessageBox(hDlg,msg,NULL,MB_OK);
                   //}
                   return (TRUE);

                case IDC_DEL_USER:        // delete the word from the user dictionary
                   GetDlgItemText(hDlg,IDC_WORD,CurWord,49);
                   if (StDelUserWord2(SplId,CurWord))
                        SetDlgItemText(hDlg,IDC_WORD_RESULT,"DELETED");
                   else SetDlgItemText(hDlg,IDC_WORD_RESULT,"NOT FOUND");
                   return (TRUE);

                case IDCANCEL:
                   EndDialog(hDlg, FALSE);
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}

/****************************************************************************
    Process messages for "StDlgFile" dialog box.
    This dialog box spell checks a text file.
****************************************************************************/

BOOL CALLBACK _export StDlgFile(HWND hDlg,UINT message,WPARAM wParam,LPARAM lParam)
{

    switch (message) {
        case WM_INITDIALOG:
            SendMessage(GetDlgItem(hDlg,IDC_READ_ONLY),BM_SETCHECK, ReadOnly, 0L);
            SetDlgItemText(hDlg,IDC_FILE,InputFile);

            SetFocus(GetDlgItem(hDlg,IDC_FILE));
            return (FALSE);

        case WM_COMMAND:
            switch (wParam) {
                case IDOK:
                   ReadOnly=(BOOL)SendMessage(GetDlgItem(hDlg,IDC_READ_ONLY),BM_GETCHECK, 0, 0L);

                   GetDlgItemText(hDlg,IDC_FILE,InputFile,63);
                   if (strlen(InputFile)==0) return TRUE;
                   if (access(InputFile,0)!=0) {
                      MessageBox(NULL,InputFile,"File Not Found",MB_OK);
                      return TRUE;
                   }

                   EndDialog(hDlg, TRUE);
                   return (TRUE);

                case IDCANCEL:
                   EndDialog(hDlg, FALSE);
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}

/*****************************************************************************
    CheckFile:
    check a file for incorrect words. Follow these steps:
      1. Open the input file. If 'ReadOnly' is not specified, open a temporary
         file for output.
      2. Read each line from the input file.  The input buffer for the 'read'
         changes after each read.  This is to preserve the previous lines
         to display on the text window, before an interactive correction is
         begun.
      3. Scan each line to scan the individual words.
      4. Spell check the extracted word.
         Check each word first without the 'ST_INTERACTIVE'.  If the word
         is misspelled,  highlight the word on the screen, and then call
         the SpellWord function again with the 'ST_INTERACTIVE' flag turned on.
         This flag instructs the SpellWord function to display a dialog box
         to the user.
*****************************************************************************/
CheckFile()
{
    FILE *iStream,*oStream;
    char *result,OutFile[128],BackupFile[128],string[100];
    int i,ExitRequest=FALSE,code;
    HCURSOR hSaveCursor;
    struct StrStResult StResult;


    /**************** Open files ****************************************/
    if (NULL==(iStream=fopen(InputFile,"rt"))) {
       MessageBox(NULL,"Can not open the input file",InputFile,MB_OK);
       return FALSE;
    }

    if (!ReadOnly) {               /* open the output file */
       strcpy(OutFile,InputFile);  /* make an output file name */
       ModifyExt(OutFile,".STT");   /* give an 'STT' extenstion to the outout file*/
       unlink(OutFile);
       if (NULL==(oStream=fopen(OutFile,"wt"))) {   /* open the output file */
          MessageBox(NULL,"Can not open the output file",OutFile,MB_OK);
          return FALSE;
       }
    }


    CurLine=MAX_LINES-1;           /* current line number in the line array */
    for (i=0;i<MAX_LINES;i++) line[i][0]=0;
    TotalWords=WrongWords=0;
    WordIndex=WordLen=LineLen=0;   /* initialize the program variables */
    CurWord[0]=0;                  /* initialize the current word */
    StClearHist2(SplId);           /* clear the history buffer (optional)*/
    hSaveCursor=SetCursor(hWaitCursor);   /* show hour glass while checking */

    NEXT_LINE:
    LinePtr=line[CurLine];         /* pointer to the current line */
    result=fgets(LinePtr,MAX_LINE_LEN,iStream);
    if (ferror(iStream)) {
        MessageBox(NULL,"Error reading the file",InputFile,MB_OK);
        return FALSE;
    }

    if (feof(iStream))  goto END_FILE;

    if (result==NULL) {
        MessageBox(NULL,"Error reading file",InputFile,MB_OK);
        return FALSE;
    }

    LineLen=lstrlen(LinePtr);
    if (LineLen>0 && LinePtr[LineLen-1]==0xA) LineLen--;  /* remove the line feed */
    LinePtr[LineLen]=0;
    LineIndex=0;                     /* number of characters checked in the current line */
    DisplayStatus("Checking...");

    //StSetFlags2(SplId,TRUE,STFLAG_NO_NUM_IN_WORD);   // example of ignoring word with numbers

    if (!ExitRequest) {              /* spell check the line if 'exit' not yet requested */
       while ((WordLen=StParseLine2(SplId,LinePtr,CurWord,&WordIndex,&LineIndex,LineLen))>0) {  /* process each word */
          if (!SpellWord2(SplId,CurWord,0,hWnd,&StResult)) {  /* mis-spelled word */

             if ((ST_EXIT|ST_ERROR)&StResult.code) {   /* exit */
                MessageBox(NULL,CurWord,LinePtr,MB_OK);
                ExitRequest=TRUE;
                break;
             }

             DisplayStatus("Correction...");
             DisplayLines();                           /* display and highlight the misspelled word */
             SetCursor(hSaveCursor);                   /* display regular cursor */

             SpellWord2(SplId,CurWord,ST_INTERACTIVE,hWnd,&StResult);
             code=StResult.code;

             if (code&(ST_EXIT|ST_ERROR)) {   /* exit */
                ExitRequest=TRUE;
                break;
             }

             if (!ReadOnly) {                 /* replace the incorrect word with a new word */
                if (StResult.replace[0]) ReplaceWord(LinePtr,CurWord,WordIndex,&LineIndex,&LineLen,StResult.replace);
             }

             if (code&(ST_IGNORE|ST_REPLACE|ST_ADD|ST_INPUT)) WrongWords++;

             DisplayStatus("Checking...");    /* resume checking */
             SetCursor(hWaitCursor);          /* show hour glass */

          }
          TotalWords++;
       }
    }

    if (!ReadOnly) {                /* write to the output */
       LinePtr[LineLen]=0xA;        /* new line character */
       LinePtr[LineLen+1]=0;
       fputs(LinePtr,oStream);      /* write to the output file */
       LinePtr[LineLen]=0;          /* restore the line */
    }

    CurLine++;
    if (CurLine>=MAX_LINES) CurLine=0;

    if (!ReadOnly || !ExitRequest) goto NEXT_LINE;

    END_FILE:

    fclose(iStream);

    if (!ReadOnly) {     /* rename the files, retain the original file as backup */
       fclose(oStream);
       strcpy(BackupFile,InputFile);  /* make the backup file name */
       ModifyExt(BackupFile,".ST");    /* give an 'ST' extenstion to the backup file*/
       unlink(BackupFile);            /* delete any existing backup file */
       if (rename(InputFile,BackupFile) || rename(OutFile,InputFile)) {
          MessageBox(NULL,"Error Renaming Files.",NULL,MB_OK);
          exit(-1);
       }
    }

    SetCursor(hSaveCursor);                    /* display regular cursor */
    wsprintf(string,"Words Checked: %ld, Number of Incorrect Words: %ld.",TotalWords,WrongWords);

    if (FileSpecified) {  // the file specified at the command line is done, display the message and exit
       MessageBox(hWnd,string,"Session Over",MB_OK);
       PostMessage(hWnd,WM_QUIT,0,0L);  // the file specified at the command line is done, now exit the program
    }
    else DisplayStatus(string);

    /************** reset the screen *****************/
    for (i=0;i<MAX_LINES;i++) line[i][0]=0;
    WordIndex=WordLen=LineLen=0;   /* initialize the program variables */
    CurWord[0]=0;                  /* initialize the current word */
    strcpy(StatusLine,"");         /* clear the status line for next display */
    DisplayLines();                /* clear the text window */

    return TRUE;
}

/*****************************************************************************
    ModifyExt:
    This function modifies the extension of a given file (argument #1) to
    as specified by the second argument.
******************************************************************************/
ModifyExt(char *InputFile,char *NewExt)
{
    #if defined(__TURBOC__)          /* compiler specific code */
        char drive[MAXDRIVE],dir[MAXDIR],FileName[MAXFILE],FileExt[MAXEXT];

        fnsplit(InputFile,drive,dir,FileName,FileExt);
        fnmerge(InputFile,drive,dir,FileName,NewExt);
    #else
        char drive[_MAX_DRIVE],dir[_MAX_DIR],FileName[_MAX_FNAME],FileExt[_MAX_EXT];
        _splitpath(InputFile,drive,dir,FileName,FileExt);
        _makepath(InputFile,drive,dir,FileName,NewExt);
    #endif

    return TRUE;
}

/******************************************************************************
    ReplaceWord:
    This routine replaces an misspelled word from a line by a replacement
    word.  The line length and current line pointer (LineIndex) variables
    are adjusted accordingly.
******************************************************************************/
ReplaceWord(char *line,char *CurWord,int WordIndex,int *LineIndex,int *LineLen,char *NewWord)
{
   int OldLen,NewLen,AdjLen,i;

   OldLen=lstrlen(CurWord);
   NewLen=lstrlen(NewWord);

   if (OldLen!=NewLen) {               /* make necessary adjustments */
      AdjLen=NewLen-OldLen;
      if ((*LineLen+AdjLen)>(MAX_LINE_LEN-1)) return FALSE;

      memmove(&line[WordIndex+NewLen],&line[WordIndex+OldLen],(*LineLen)-WordIndex-OldLen+1);

      *LineLen=(*LineLen)+AdjLen;      /* adjust the line length */
      *LineIndex=(*LineIndex)+AdjLen;  /* adjust the current word pointer within the current line*/
   }

   for(i=0;i<NewLen;i++) line[WordIndex+i]=NewWord[i]; /* substitute the new word */

   return TRUE;
}

/******************************************************************************
    InitWin:
    Initialize the demo window variables.
*******************************************************************************/
InitWin()
{
    if (NULL==(hDC=GetDC(hWnd))) {      /* allocate a class DC */
         MessageBox(NULL,"Unable to allocate a Display Context.",NULL,MB_OK);
         return FALSE;
    }

    if (NULL==(hRegFont=CreateFont(0,
                                     0,
                                     0,
                                     0,
                                     0,
                                     FALSE,FALSE,FALSE,
                                     ANSI_CHARSET,
                                     OUT_DEFAULT_PRECIS,
                                     CLIP_DEFAULT_PRECIS,
                                     DEFAULT_QUALITY,
                                     FIXED_PITCH|FF_ROMAN,
                                     "Courier New"))) {   /* create a fixed pitch font */
         MessageBox(NULL,"Unable to create the font.",NULL,MB_OK);
    }

    if (NULL==SelectObject(hDC,hRegFont)) {   /* select the newly created font */
         MessageBox(NULL,"Unable to select the font.",NULL,MB_OK);
    }

    if (!GetTextMetrics(hDC,&TextMet)) {     /* get text metric for the newly created font */
         MessageBox(NULL,"Unable to get text metric for the font.",NULL,MB_OK);
    }

    TextHeight=TextMet.tmHeight+TextMet.tmExternalLeading;


    SetBkMode(hDC,OPAQUE);                   /* enable opaque printing of text */

    GetWinDimension();                        /* get window dimensions */

    ValidateRect(hWnd,NULL);               /* No need to paint right now */
    return TRUE;
}

/******************************************************************************
    RepaintWin:
    This routine is called when a paint message is received.
    This routine repaint the entire client area.
*******************************************************************************/
RepaintWin()
{
    PAINTSTRUCT PaintData;

    BeginPaint(hWnd,&PaintData);
    EndPaint(hWnd,&PaintData);

    GetWinDimension();     /* get window dimensions */

    DisplayStatus(StatusLine);   /* PaintFlag the status line */

    /********** draw horrizontal lines *************/
    MoveToEx(hDC,TextRect.left,TextRect.top-2,NULL);        /* draw line over the text window */
    LineTo(hDC,TextRect.right,TextRect.top-2);

    MoveToEx(hDC,TextRect.left,TextRect.bottom+2,NULL);     /* draw line under the text window */
    LineTo(hDC,TextRect.right,TextRect.bottom+2);

    MoveToEx(hDC,TextRect.left,StatusLineY-2,NULL);         /* draw line over the status line */
    LineTo(hDC,TextRect.right,StatusLineY-2);


    DisplayLines();                                   /* paint the text lines */

    return TRUE;
}

/******************************************************************************
    GetWinDimension:
    The routine gets the following dimensions of the current window:
       WinWidth in number of character columns.
       WinHeight in number of character rows.
*******************************************************************************/
int GetWinDimension()
{
    int HeightAdj=0;

    GetClientRect(hWnd,(LPRECT) &rect);

    WinWidth=(rect.right-rect.left)/(TextMet.tmAveCharWidth)-1;
    WinHeight=((rect.bottom-rect.top)/TextHeight)-1;

    WinHeight--;                        /* last line may be used for PaintFlaging the status line */
    if (WinHeight==0) WinHeight=1;      /* minimum window height */
    if (WinHeight>MAX_LINES) {          /* center the text are in the window */
        HeightAdj=(WinHeight-MAX_LINES)*(TextMet.tmHeight+TextMet.tmExternalLeading);
        WinHeight=MAX_LINES;}
    if (WinWidth<=0)  WinWidth=1;       /* minimum window width */


    TextRect.left=rect.left+TextMet.tmAveCharWidth/2;
    TextRect.right=TextRect.left+TextMet.tmAveCharWidth*WinWidth;

    TextRect.top=rect.top+TextHeight/2+HeightAdj/2;
    TextRect.bottom=TextRect.top+TextHeight*WinHeight;

    StatusLineY=rect.bottom-(TextHeight*3)/2;

    return TRUE;
}

/*****************************************************************************
    DisplayLines:
    Display the text lines being checked.
*****************************************************************************/
int DisplayLines()
{
    int i,row,BeginCol=0,StartRow;
    char SpaceLine[MAX_LINE_LEN];  // 20160517: changed from 160 to MAX_LINE_LEN which is now increased to 2000

    for (i=0;i<WinWidth;i++) SpaceLine[i]=' ';
    SpaceLine[i]=0;

    /******* display the current line and previous 2 lines in the text window */
    row=0;
    StartRow=MAX_LINES-WinHeight;
    i=CurLine+1;

    while(TRUE) {
       if (i>=MAX_LINES) i=0;

       if (i==CurLine) {    /* determine the starting position of the current line to display */

          /* the BeginCol should be such that the entire current word can
             be seen in the text window */

          if (LineLen<=WinWidth) BeginCol=0;
          else BeginCol=(WordIndex+WordLen+5)-WinWidth;
          if (BeginCol<0) BeginCol=0;
       }

       if (row>=StartRow) WriteText(row-StartRow,0,&line[i][BeginCol],lstrlen(line[i])-BeginCol,FALSE);
       if (i==CurLine) {    /* highlight the current word */
           if (row>=StartRow) WriteText(row-StartRow,WordIndex-BeginCol,CurWord,WordLen,TRUE);
           break;
       }
       i++;
       row++;
    }
    return TRUE;
}

/******************************************************************************
    WriteText:
    Write a line of text the given location.
******************************************************************************/
int WriteText(int row,int col,LPSTR line,int len,BOOL ReverseColor)
{
    RECT rect;
    DWORD TextColor,TextBkColor;

    rect.top=TextRect.top+row*TextHeight;
    rect.bottom=rect.top+TextHeight;
    rect.left=TextRect.left+col*TextMet.tmAveCharWidth;
    rect.right=TextRect.right;

    if (ReverseColor) {             /* highlight colors */
        TextColor=GetTextColor(hDC);
        TextBkColor=GetBkColor(hDC);
        SetBkColor(hDC,(COLORREF)0x0000FF);
        SetTextColor(hDC,(COLORREF)0x0);

        TextOut(hDC,rect.left,rect.top,line,len);

        SetTextColor(hDC,TextColor);
        SetBkColor(hDC,TextBkColor);
    }
    else ExtTextOut(hDC,rect.left,rect.top,ETO_OPAQUE|ETO_CLIPPED,&rect,line,len,NULL);

    return TRUE;
}

/******************************************************************************
    DisplayStatus:
    Display the status line.
******************************************************************************/
DisplayStatus(LPSTR status)
{
    RECT rect;

    lstrcpy(StatusLine,status);       /* copy to the global variable */

    rect.top=StatusLineY;
    rect.bottom=StatusLineY+TextHeight;
    rect.left=TextRect.left;
    rect.right=TextRect.right;

    ExtTextOut(hDC,TextRect.left,StatusLineY,ETO_OPAQUE,&rect,StatusLine,lstrlen(StatusLine),NULL);

    return TRUE;
}

/******************************************************************************
    OurPrintf:
    This routine formats and display a given set of arguments.  The format
    is given by the first argument.  The argument 2 to n contain the
    arguments for the specified format.  The function uses MessageBox to
    display the formatted str.
******************************************************************************/
int OurPrintf(LPSTR fmt,...)
{
    LPSTR ArgStart;
    char str[1000];
    HWND hWnd;

    ArgStart=(LPSTR) &fmt;  // pointer to first argument
    ArgStart=&ArgStart[4];  // go past the first argument
    wvsprintf(str,fmt,ArgStart);

    if (NULL!=(hWnd=FindWindow("DBWin",NULL))) { // debug window open
        if (NULL!=(hWnd=GetWindow(hWnd,GW_CHILD))) {    // get the edit window
           int len=lstrlen(str);
           str[len]=0xd;
           str[len+1]=0xa;
           str[len+2]=0;
           #if defined (WIN32)
              SendMessage(hWnd,EM_SETSEL,32767,32767);  // show the last line
              SendMessage(hWnd,EM_REPLACESEL,0,(DWORD)(LPSTR)str);
              SendMessage(hWnd,EM_SCROLLCARET,0,0);     // scroll caret into view
           #else
              SendMessage(hWnd,EM_SETSEL,0,MAKELPARAM(32767,32767));  // show the last line
              SendMessage(hWnd,EM_REPLACESEL,0,(DWORD)(LPSTR)str);
              SendMessage(hWnd,EM_SETSEL,0,MAKELPARAM(32767,32767));  // show the last line
           #endif
        }
    }
    else MessageBox(NULL,str,NULL,MB_OK);
    return TRUE;
}
